package vo

import "github.com/gogf/gf/v2/frame/g"

//银盛公共请求参数
type PayCommonReq struct {
	Method     string `json:"method"`      //接口名称
	PartnerId  string `json:"partner_id"`  //银盛支付开设的服务商商户号
	Timestamp  string `json:"timestamp"`   //发送请求的时间
	Charset    string `json:"charset"`     //商户网站使用的编码格式，
	SignType   string `json:"sign_type"`   //报文签名算法
	Sign       string `json:"sign"`        //签名字符串,再用Base64编码
	NotifyUrl  string `json:"notify_url"`  //交易成功异步通知到商户的后台地址
	Version    string `json:"version"`     //交易类型
	BizContent string `json:"biz_content"` //业务请求参数的集合
}

//银盛业务请求参数
type PayBusReq struct {
	OutTradeNo     string `json:"out_trade_no",omitempty`    //商户系统生成的订单号
	Shopdate       string `json:"shopdate",omitempty`        //商户系统的交易发生日期格式yyyyMMdd 20180525
	Subject        string `json:"subject",omitempty`         //订单备注
	TotalAmount    string `json:"total_amount",omitempty`    //该笔订单的资金总额
	Currency       string `json:"currency",omitempty`        //默认CNY
	SellerId       string `json:"seller_id",omitempty`       //收款商户号
	TimeoutExpress string `json:"timeout_express",omitempty` //设置未付款交易的超时时间
	BusinessCode   string `json:"business_code",omitempty`   //业务代码

}
type PayZfBusReq struct {
	PayBusReq
	BuyerId string `json:"buyer_id"` //买家支付宝用户ID
}
type PayWxBusReq struct {
	PayBusReq
	SubOpenid string `json:"sub_openid"` //用户在商户appid下的唯一标识
	IsMinipg  string `json:"is_minipg"`  //微信小程序支付：1
	Appid     string `json:"appid"`      //商户在微信侧申请的小程序或公众号APPID
}

type WxPayResp struct {
	Sign                          string     `json:"sign"`                              //签名字符串  Base64编码
	YsepayOnlineWeixinPayResponse BusPayResp `json:"ysepay_online_weixin_pay_response"` //业务响应参数的集合，最大长度不限
}
type ZfbPayResp struct {
	Sign                            string     `json:"sign"`                                //签名字符串  Base64编码
	YsepayOnlineAlijsapiPayResponse BusPayResp `json:"ysepay_online_alijsapi_pay_response"` //业务响应参数的集合，最大长度不限
}

type BusPayResp struct {
	Code             string  `json:"code"`               //响应代码
	Msg              string  `json:"msg"`                //响应代码描述
	OutTradeNo       string  `json:"out_trade_no"`       //商户系统生成的订单号
	TradeNo          string  `json:"trade_no"`           //银盛支付交易流水号
	TradeStatus      string  `json:"trade_status"`       //交易状态，成功状态的值： TRADE_SUCCESS参考附录8.1
	TotalAmount      string  `json:"total_amount"`       //该笔订单的资金总额
	Currency         string  `json:"currency"`           //交易币种 默认CNY
	ExtraCommonParam string  `json:"extra_common_param"` //公用回传参数 商户自定义数据域，原样返回
	JsapiPayInfo     string  `json:"jsapi_pay_info"`     //Json格式字符串，作用于原生态的js支付时的参数
	IsDiscount       string  `json:"is_discount"`        //是否参与优惠，Y表示参与，N表示不参与
	TotalDisCount    float64 `json:"total_discount"`     //参考总优惠金额 100．00
	ChannelSendSN    string  `json:"channel_send_sn"`    //发往渠道流水号
}

// 银盛异步通知
type NotifyPayReq struct {
	g.Meta      `path:"/notifyPay" method:"post"`
	SignType    string `json:"sign_type"`    //签名类型
	Sign        string `json:"sign"`         //签名字符串
	NotifyType  string `json:"notify_type"`  //通知类型
	NotifyTime  string `json:"notify_time"`  //发送请求的时间
	OutTradeNo  string `json:"out_trade_no"` //商户生成的订单号
	TradeStatus string `json:"trade_status"` //交易目前所处的状态 成功状态的值： TRADE_SUCCESS
}
